/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  Gac - create binary data file
 *
 *  This program reads a formatted ascii file containing galaxy
 *  information and produces a binary file.
 */

#include <fcntl.h>
#include <stdio.h>
#include <gax.h>
#include <math.h>

/*
 *          1         2         3         4         5         6         7         8         9         0         1
 * 12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
 * ----- ------ ------- ----- ----- ----- ---- --- ------ ------ ---- -------- ------------ --------- ------ ------ ------
 *     1 010101 +100101 432.1 123.1 12345 12.1 121 N1001a I2001a M101 UA30001a G+11-22-333a E151-251a K1601a A1701a V1801a
 *
 */

a2d (ubyte_t *p, gxd_t *d)
{
	 long_t	 n = 0;
	ubyte_t	*b = p;			/* pointer into record */
	fp64_t	 R, D, X, Y;

	memset (d, 0, sizeof (gxd_t));

	if (*(b+63-1) == 'A') *(b+69-1) = 'F';		/* UGCA -> UGC F */

	p = b +   7 - 1; if (a2i_ra  (&p, &d->raj_n)) goto ERR;
	p = b +  14 - 1; if (a2i_dec (&p, &d->dcj_n)) goto ERR;
	p = b +  22 - 1; if (a2i_maj (&p, &d->maj_n)) goto ERR;
	p = b +  28 - 1; if (a2i_min (&p, &d->min_n)) goto ERR;
#ifdef GX_Vel
	p = b +  34 - 1; if (a2i_vel (&p, &d->vel_n)) goto ERR;
#endif
	p = b +  40 - 1; if (a2i_mv  (&p, &d-> mv_n)) goto ERR;
	p = b +  45 - 1; if (a2i_pa  (&p, &d-> pa_n)) goto ERR;

	/*
	 *	character location is one more than start of field
	 *	because we ignore the catalog designator letter.
	 */
#ifdef GX_PGC
	p = b +   1 - 1; if (a2b_pgc (&p, &d->pgc_a)) goto ERR;
	if (d->pgc_a) n = 1;
#endif
#ifdef GX_NGC
	p = b +  50 - 1; if (a2b_ngc (&p, &d->ngc_a)) goto ERR;
	if (d->ngc_a) n = 1;
#endif
#ifdef GX_IC
	p = b +  57 - 1; if (a2b_ic  (&p, &d-> ic_a)) goto ERR;
	if (d->ic_a) n = 1;
#endif
#ifdef GX_M
	p = b +  64 - 1; if (a2b_m   (&p, &d->  m_a)) goto ERR;
	if (d->m_a) n = 1;
#endif
#ifdef GX_UGC
	p = b +  70 - 1; if (a2b_ugc (&p, &d->ugc_a)) goto ERR;
	if (d->ugc_a) n = 1;
#endif
#ifdef GX_MGC
	p = b +  78 - 1; if (a2b_mcg (&p, &d->mcg_a)) goto ERR;
	if (d->mcg_a) n = 1;
#endif
#ifdef GX_ESO
	p = b +  91 - 1; if (a2b_eso (&p, &d->eso_a)) goto ERR;
	if (d->eso_a) n = 1;
#endif
#ifdef GX_MK
	p = b + 101 - 1; if (a2b_mk  (&p, &d-> mk_a)) goto ERR;
	if (d->mk_a) n = 1;
#endif
#ifdef GX_ARP
	p = b + 108 - 1; if (a2b_arp (&p, &d->arp_a)) goto ERR;
	if (d->arp_a) n = 1;
#endif
#ifdef GX_VV
	p = b + 115 - 1; if (a2b_vv  (&p, &d-> vv_a)) goto ERR;
	if (d->vv_a) n = 1;
#endif

	R = d->raj_n; R /= 3600;
	D = d->dcj_n; D /= 3600;
	d->con_n = con_fnd_nam (2000.0, R, D);

	/*
	 *	Calculate the surface brightness if info available
	 */
	if (d->maj_n && d->min_n && d->mv_n)
		d->sb_n = d->mv_n + 25.0 * log10 (3.141592654*d->maj_n*d->min_n/400.0);

	return (n);

ERR:
	a2i_a2e (b, p);
	return (0);
}

main (long_t ac, byte_t *av [])
{
	FILE	*fdi_p;
	ubyte_t	 buf_a [512];
	 long_t	 fdo_n;
	  gxd_t  gxd_p [1];

	if (ac == 1) help ();
	if (!strcmp (av [1], "-V")) version ();
	if (ac != 3) help ();

	if (!(fdi_p = fopen (av [1], "r"))) {
		printf ("unable to open input file %s\n", av [1]);
		exit (-1);
	}
	if (!(fdo_n = gxd_open (av [2], O_CREAT | O_WRONLY | O_TRUNC, 0666))) {
		printf ("unable to open output file %s\n", av [2]);
		exit (-1);
	}
	while (fgets (buf_a, sizeof (buf_a), fdi_p)) {
		buf_a [strlen (buf_a)-1] = 0;				/* remove \n */
		if (a2d (buf_a, gxd_p))
		    gxd_write (fdo_n, gxd_p, 1);
	}
	gxd_close (fdo_n);
	fclose    (fdi_p);
}

help ()
{
	printf ("usage: gac [-V] input output\n");
	printf ("  where:\n");
	printf ("       -V - print version and catalog information\n");
	printf ("    input - input ascii catalog data\n");
	printf ("   output - output .gax binary data file\n");
	exit (0);
}

version ()
{
		gax_ver ("gac");
	    exit (0);
}

