/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define	CON

/*
 *	This file is based on the epoch J2000.0
 */
#include <math.h>
#include <mpa.h>
#include <gax_con.h>

/*
 *	con_i2a_nam - convert constellation index to ascii constellation name
 */
ubyte_t	*con_i2a_nam (UB n)
{
	return ((n <= 88) ? con_nam_a [n] : con_nam_a [0]);
}

/*
 *	con_a2i_nam - convert ascii constellationname to constellation index
 */
ubyte_t	 con_a2i_nam (UB *p)
{
	long_t	i;

	for (i = 1; i <= 88; i++)
		if (!strncmp (p, con_nam_a [i], 3))
			return (i);

	return (0);
}

#define	CONVH	0.2617993878
#define	CONVD	0.01745329251994

static fp64_t ep1 = 0.0, ep2 = 0.0;
static fp64_t r [3] [3];

/*
 *	con_rotate - rotate coordinate system between epochs
 */
void_t	 con_rotate (fp64_t epoch1, fp64_t epoch2)
{
	fp64_t	csr, t, st, a, b, c, x, cosa, cosb, cosc, sina, sinb, sinc;

	ep1 = epoch1;
	ep2 = epoch2;

	csr = CONVD / 3600.0;
	t   = 0.001 * (epoch2 - epoch1);
	st  = 0.001 * (epoch1 - 2000.0);

	x = 23062.181 + st * (139.656 + 0.139 * st);
	a = csr * t * (x + t * ( 30.188 - 0.344 * st + 17.998 * t));
	b = csr * t * (x + t * (109.468 - 0.066 * st + 18.203 * t));
	
	c = csr * t * (20043.109 - st * ( 85.330 + 0.217 * st) +
	                            t * (-42.665 - 0.217 * st - 41.833 * t));

	sina = sin (a); sinb = sin (b); sinc = sin (c);
	cosa = cos (a); cosb = cos (b); cosc = cos (c);

	r [0][0] =  (cosa * cosb * cosc) - (sina * sinb);
	r [0][1] = -(cosa * sinb) - (sina * cosb * cosc);
	r [0][2] = -(cosb * sinc);
	r [1][0] =  (sina * cosb) + (cosa * sinb * cosc);
	r [1][1] =  (cosa * cosb) - (sina * sinb * cosc);
	r [1][2] = -(sinb * sinc);
	r [2][0] =  (cosa * sinc);
	r [2][1] = -(sina * sinc);
	r [2][2] =  (cosc);
}

/*
 *	con_precess - precess coordinates from epoch1 to epoch2
 */
void_t	 con_precess (fp64_t	 ra1,		/* RA  at epoch E1 */
					  fp64_t	 dec1,		/* DEC at epoch E1 */
					  fp64_t	 epoch1,	/* Epoch 1 */
					  fp64_t	 epoch2,	/* Epoch 2 */
					  fp64_t	*ra2,		/* RA  at epoch 2 */
					  fp64_t	*dec2)		/* DEC at epoch 2 */
{
	fp64_t	 x1 [3], x2 [3], cosd;
	long_t	 i, j;

	         cosd = cos (dec1);
	x1 [0] = cosd * cos (ra1);
	x1 [1] = cosd * sin (ra1);
	x1 [2] =        sin (dec1);

	if (ep1 != epoch1 || ep2 != epoch2) con_rotate (epoch1, epoch2);

	for (i = 0; i < 3; i++) {
		x2 [i] = 0.0;
		for (j = 0; j < 3; j++)
			x2 [i] += r [i][j] * x1 [j];
	}

	if ((*ra2 = atan2 (x2 [1], x2 [0])) < 0.0)
		 *ra2 += TWOPI;
	*dec2 = asin (x2 [2]);
}

/*
 *	con_fnd_nam - find constellation name from RA and Dec in Epoch
 *
 *	Note: that this has to be done in the Epoch 1875 which is when
 *	the constellation boundaries were defined and were aligned with
 *	circles of RA and declination.  This is no longer the case due
 *	to precession.
 */
ubyte_t	 con_fnd_nam (fp64_t e,	/* epoch in years */
					  fp64_t r,	/* RA  in seconds of time at epoch */
					  fp64_t d)	/* DEC in seconds of arc  at epoch */
{
	fp64_t	 R, D;	/* RA/DEC at epoch 1875.0 */
	long_t	 i = 0;

#ifdef	SECS
	r = (r / 3600.0) * CONVH;
	d = (d / 3600.0) * CONVD;
#else
	r *= CONVH;
	d *= CONVD;
#endif

	con_precess (r, d, e, 1875.0, &R, &D);

	if (R <   0.0 ) R += TWOPI;
	if (R >= TWOPI) R -= TWOPI;

	R /= CONVH;
	D /= CONVD;

DEC:while (i < CON_POS_N && con_pos_a [i].dec_f >  D) i++;
RAU:while (i < CON_POS_N && con_pos_a [i].rau_f <= R) i++;
RAL:while (i < CON_POS_N && con_pos_a [i].ral_f >= R) i++;

	if (i >= CON_POS_N) return (0);

	if (R >= con_pos_a [i].ral_f && R <= con_pos_a [i].rau_f &&
									D >  con_pos_a [i].dec_f)
		return (con_pos_a [i].con_n);

	if (R >= con_pos_a [i].rau_f) goto RAU;

	return (0);
}

#ifdef	TEST

main ()
{
	printf ("And (01): %3s (%02d)\n", con_i2a_nam ( 1), con_a2i_nam ("And"));
	printf ("Gem (38): %3s (%02d)\n", con_i2a_nam (38), con_a2i_nam ("Gem"));
	printf ("Vul (88): %3s (%02d)\n", con_i2a_nam (88), con_a2i_nam ("Vul"));

	printf ("UMa: %3s\n", con_i2a_nam (con_fnd_nam (1950.0, 09.0000, 65.0000)));
	printf ("Aqr: %3s\n", con_i2a_nam (con_fnd_nam (1950.0, 23.5000,-20.0000)));
	printf ("Ori: %3s\n", con_i2a_nam (con_fnd_nam (1950.0,  5.1200,  9.1200)));
	printf ("Com: %3s\n", con_i2a_nam (con_fnd_nam (1950.0, 12.8888, 22.0000)));
	printf ("CrA: %3s\n", con_i2a_nam (con_fnd_nam (1950.0, 19.0000,-40.0000)));
	printf ("Men: %3s\n", con_i2a_nam (con_fnd_nam (1950.0, 06.2222,-81.1234)));
}

#endif

