/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <mpa.h>
#include <gax_tim.h>

/*
 *	Taken from the book: Astronomical Algorithms, Meeus
 */

/*
 *	Calculate Julian Day (JD) from Day, Month, Year
 *		D - Day of Month  [1-31]
 *		M - Month of Year [1-12]
 *		Y - Year
 */
fp64_t tim_DMY2JD (fp64_t D, UL M, UL Y)
{
	SL	a, b, m, y;

	if (M <= 2) { Y--; M += 12; }

	/*
	 *	Need to check for Julian Calendar
	 */
	a = Y / 100;
	b = 2 - a + (a / 4);

ZERO:
	y = 365.25 * (Y + 4716);
	m = 30.601 * (M + 1);

	return (y + m + D + b - 1524.5);
}

/*
 *	Calculate Mean Sidereal Time at Greenwich at 0hours UT in Seconds of Time
 *		JD - Julian Day
 */
fp64_t tim_JD2GMST0 (fp64_t JD)
{
	fp64_t	 T, ST;

	T = (JD - 2451545.0) / 36525.0;

	ST = 24110.54841 + (8640184.812866 + (0.093104 * T)) * T;

	ST = ST - (T * T * T * 0.0000062);

	while (ST < 0.0)
		ST += 86400.0;

	while (ST > 86400.0)
		ST -= 86400.0;

	return (ST);
}

/*
 *	Calculate Transit Time of RA in seconds of time
 *		ra    - RA in seconds of time
 *		lon   - Longitude in seconds of arc
 *		GMST0 - Greenwich Mean Sidereal Time at 0h UT in seconds of time
 */
fp64_t tim_transit (SL ra, SL lon, fp64_t GMST0)
{
	fp64_t	 L = lon * 86400 / (360.0 * 3600.0);

	L = ra - L - GMST0;

	while (L  <     0.0) L += 86400.0;
	while (L >= 86400.0) L -= 86400.0;

	return (L);
}

/*
 *	NOT EXACTLY WORKING
 *
 *	Calculate Rise Time of RA in seconds of time
 *		ra    - RA in seconds of time
 *		lon   - Longitude in seconds of arc
 *		GMST0 - Greenwich Mean Sidereal Time at 0h UT in seconds of time
 *		h0    - Altitude in seconds of arc
 *		lat   - Latitude in seconds of arc
 *		dec   - Declination in seconds of arc
 */
fp64_t tim_rise (SL ra, SL lon, fp64_t GMST0, SL h0, SL lat, SL dec)
{
	fp64_t	 R, H0, cosH0;
	fp64_t	 T = tim_transit (ra, lon, GMST0);
	fp64_t	 sinh0  = h0  * TWOPI / (360.0 * 3600.0);
	fp64_t	 sinlat = lat * TWOPI / (360.0 * 3600.0);
	fp64_t	 sindec = dec * TWOPI / (360.0 * 3600.0);
	fp64_t	 coslat = sinlat;
	fp64_t	 cosdec = sindec;

	sinh0  = sin (sinh0);
	sinlat = sin (sinlat);
	sindec = sin (sindec);
	coslat = cos (coslat);
	cosdec = cos (cosdec);

	cosH0  = (sinh0 - sinlat * sindec) / (coslat * cosdec);

#if 0
	putchar ('\n');
	printf ("   ra: %d\n", ra);
	printf ("  dec: %d\n", dec);
	printf ("  lat: %d\n", lat);
	printf ("  lon: %d\n", lon);
	printf ("   h0: %d\n", h0);
	printf ("sinh0 : %.6lf\n", sinh0 );
	printf ("sinlat: %.6lf\n", sinlat);
	printf ("sindec: %.6lf\n", sindec);
	printf ("coslat: %.6lf\n", coslat);
	printf ("cosdec: %.6lf\n", cosdec);
	printf ("cosH0 : %.6lf\n", cosH0 );
#endif

	if (cosH0 < -1.0 || cosH0 > 1.0) return (-1.0);

	H0 = acos (cosH0);

	H0 = H0 / TWOPI * 360.0 * 3600.0;

	R = T - H0;

	while (R  <     0.0) R += 86400.0;
	while (R >= 86400.0) R -= 86400.0;

	return (R);
}

/*
 *	NOT WORKING AT ALL
 *
 *	Calculate Set Time of RA in seconds of time
 *		r - RA in seconds of time
 *		l - Longitude in seconds of arc
 *		GMST0 - Greenwich Mean Sidereal Time at 0h UT in seconds of time
 */
fp64_t tim_set (SL r, SL l, fp64_t GMST0)
{
	fp64_t	 L = l * 86400 / (360.0 * 3600.0);

	L = r - L - GMST0;

	return (0.0);
}

#ifdef TEST

main ()
{
	fp64_t	JD, GMST0;

	printf ("(2443259.50) %.2lf\n", tim_DMY2JD (26.00,  4, 1977));
	printf ("(2436115.50) %.2lf\n", tim_DMY2JD ( 4.00, 10, 1957));
	printf ("(2436116.31) %.2lf\n", tim_DMY2JD ( 4.81, 10, 1957));

	JD    = tim_DMY2JD   (1.0, 1, 1995);
	GMST0 = tim_JD2GMST0 (JD);
	printf ("JD=%.2lf  GMST0=%.5lf\n", JD, GMST0);

	printf ("TT=%.2lf\n", tim_transit (20*3600,+75*3600,GMST0));
}

#endif

